#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ERR_OK          0
#define ERR_NOK        -1

#define TYPE_NOK        0
#define TYPE_BIN        1
#define TYPE_IMG        2

#define IDENT_UNKNOWN   0x00
#define IDENT_OLD_BAS   0x20
#define IDENT_OLD_PAS   0x21
#define IDENT_OLD_BIN   0x26
#define IDENT_NEW_BAS   0x70
#define IDENT_NEW_PAS   0x71
#define IDENT_EXT_BAS   0x72
#define IDENT_EXT_PAS   0x73
#define IDENT_NEW_BIN   0x76
#define IDENT_PC1211    0x80
#define IDENT_PC1500    0x0A
#define IDENT_PC15_BIN  0xA0
#define IDENT_PC15_BAS  0xA1

#define ORDER_STD       0
#define ORDER_INV       1

#define MODE_B22        0
#define MODE_B21        1
#define MODE_B20        2
#define MODE_B19        3
#define MODE_B17        4
#define MODE_B16        5
#define MODE_B14        6
#define MODE_B13        7

#define BASE_FREQ1      4000
#define BASE_FREQ2      2500

#define CHAR_SPACE      0x20
#define CHAR_DOT        0x2E
#define CHAR_COLON      0x3A
#define CHAR_SLASH      0x5C
#define CHAR_UNDERSCORE 0x5F

#define TOKEN_NB        5
static char* Token[] = { "T:", "PC:", "A:", "S:", "N:" } ;

typedef unsigned char    u_char;    /*  8 bits */
typedef unsigned short   u_short;   /* 16 bits */
typedef unsigned long    u_long;    /* 32 bits */

typedef struct {
    FILE*   ptrFd ;
    u_long  ident ;
    u_long  count ;
    u_long  sum ;
} FileInfo ;

typedef struct {
    u_long  nb1 ;
    u_long  nb2 ;
} ModeInfo ;

static ModeInfo Mode[] = {
    { 6, 6 }, { 5, 6 }, { 5, 5 }, { 4, 5 },
    { 1, 6 }, { 1, 5 }, { 1, 3 }, { 1, 2 }
} ;

static char* bit[] = {
    "\255\255\0\0\255\255\0\0\255\255\0\0\255\255\0\0",
    "\255\0\255\0\255\0\255\0\255\0\255\0\255\0\255\0"
} ;

static u_long CodeOld[] = {
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x11,0x14,0x12,0x15,0x19,0x16,0x1F,0x11,
    0x30,0x31,0x37,0x35,0x1B,0x36,0x4A,0x38,
    0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,
    0x48,0x49,0x1D,0x1C,0x33,0x34,0x32,0x13,
    0x1E,0x51,0x52,0x53,0x54,0x55,0x56,0x57,
    0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,
    0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,
    0x68,0x69,0x6A,0x11,0x11,0x11,0x39,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,
    0x19,0x11,0x1A,0x11,0x11,0x11,0x11,0x11,
    0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
} ;


void CvShortToStringI (u_short value,
                       char*   ptrStr)
{
    u_short tmp ;

    /* Convert the short value into a String with msb first (INTEL) */
    tmp = value & 0xFF ;
    *ptrStr ++ = (char) tmp ;
    tmp = value >> 8 ;
    *ptrStr ++ = (char) tmp ;
    *ptrStr = 0 ;
}

void CvLongToStringI (u_long value,
                      char*  ptrStr)
{
    u_long  tmp ;

    /* Convert the long value into a String with msb first (INTEL) */
    tmp = value & 0xFF ;
    *ptrStr ++ = (char) tmp ;
    tmp = (value >> 8) & 0xFF ;
    *ptrStr ++ = (char) tmp ;
    tmp = (value >> 16) & 0xFF ;
    *ptrStr ++ = (char) tmp ;
    tmp = value >> 24 ;
    *ptrStr ++ = (char) tmp ;
    *ptrStr = 0 ;
}


int WriteStringToFile (char*  ptrStr,
                       FILE** ptrFd)
{
    int     error ;

    error = fputs (ptrStr, *ptrFd) ;
    if (error == EOF) {
        printf ("ERROR : Can't write in the file\n") ;
        error = ERR_NOK ;
    }
    else {
        error = ERR_OK ;
    }
    return (error);
}


int WriteLongToFile (u_long value,
                     FILE** ptrFd)
{
    char    str[10] ;
    int     ii ;
    int     error ;

    CvLongToStringI (value, str) ;

    for ( ii = 0 ; ii < 4 ; ii++  ) {
        error = fputc (str[ii], *ptrFd) ;
        if (error == EOF) {
            printf ("ERROR : Can't write in the file\n") ;
            error = ERR_NOK ;
            break ;
        }
        else
            error = ERR_OK ;
    }
    return (error);
}


int WriteShortToFile (u_long value,
                      FILE** ptrFd)
{
    char    str[10] ;
    int     ii ;
    int     error ;

    CvShortToStringI (value, str) ;

    for ( ii = 0 ; ii < 2 ; ii++ ) {
        error = fputc (str[ii], *ptrFd) ;
        if (error == EOF) {
            printf ("ERROR : Can't write in the file\n") ;
            error = ERR_NOK ;
            break ;
        }
        else
            error = ERR_OK ;
    }
    return (error);
}


int ReadFileLength (u_long* ptrLen,
                    FILE**  ptrFd)
{
    long    nbByte ;
    int     error ;

    do {
        *ptrLen = 0 ;

        /* Seek to the end of the source file */
        error = fseek (*ptrFd, 0, SEEK_END) ;
        if (error != ERR_OK) {
            printf ("ERROR : Can't seek the file\n") ;
            break ;
        }

        /* Get the length of the source file */
        nbByte = ftell (*ptrFd) ;
        if (nbByte == ERR_NOK) {
            printf ("ERROR : Can't ftell the file\n") ;
            error = ERR_NOK ;
            break ;
        }

        /* Seek to the begining of the source file */
        error = fseek (*ptrFd, 0, SEEK_SET) ;
        if (error != ERR_OK) {
            printf ("ERROR : Can't seek the file\n") ;
            break ;
        }

        if (nbByte == 0) {
            printf ("ERROR : Source file is empty\n") ;
            error = ERR_NOK ;
            break ;
        }

        *ptrLen = nbByte ;

    } while (0) ;
    return (error);
}


int LengthAndIdentOfBinWav (u_long   type,
                            u_long   pcId,
                            u_long   nbByte,
                            u_long   nbSync,
                            u_long*  ptrIdent,
                            u_long*  ptrSamp)
{
    u_long  ident ;
    u_long  nbSamp ;
    int     error ;

    do {
        error = ERR_OK ;
        switch (pcId) {
        case 1500 :
            if (type == TYPE_BIN) {
                /* Footer = 3 bytes : sum, 0x55 */
                ident = IDENT_PC15_BIN ;
                nbSamp = 3 ;
            }
            else {
                /* Footer = 4 bytes : 0xFF, sum, 0x55 */
                ident = IDENT_PC15_BAS ;
                nbSamp = 4 ;
            }

            /* Header = 42, Lead = nbByte + 2 * (nbByte / 80) */
            /* One Byte = 22 bits, each Bit = 16 Samples      */
            /* One Sync = one Bit = 16 Samples (2500 Hz)      */
            /* Ident  = 11 bits, 3 gaps of 70 bits            */

            nbSamp = nbSamp + 42 + nbByte + 2 * (nbByte / 80) ;
            nbSamp = nbSamp * 22 * 16 ;
            nbSamp = nbSamp + (nbSync * 16) ;
            nbSamp = nbSamp + (221 * 16) ;
            break ;

        case 1245 :
        case 1251 :
            if (type == TYPE_BIN) {
                ident = IDENT_OLD_BIN ;
                nbSamp = 1 + (2 * 9) ;  /* Header = 19 Byte */
            }
            else {
                ident = IDENT_OLD_BAS ;
                nbSamp = 1 + 9 ;        /* Header = 10 Byte */
            }

            /* Footer = 1, Lead = nbByte + (nbByte / 8)  */
            /* One Byte = 19 bits, each Bit = 16 Samples */
            /* One Sync = one Bit = 16 Samples (4000 Hz) */

            nbSamp = nbSamp + 1 + nbByte + (nbByte / 8) ;
            nbSamp = nbSamp * 19 * 16 ;
            nbSamp = nbSamp + (nbSync * 16) ;
            break ;

        case 1261 :
        case 1350 :
        case 1360 :
        case 1401 :
        case 1402 :
        case 1403 :
        case 1450 :
        case 1475 :
            if (type == TYPE_BIN) {
                ident = IDENT_NEW_BIN ;
                nbSamp = (1 + (2 * 9)) * 16 * 16 ;  /* Header = 19 Byte16 */
            }
            else {
                if ( (pcId == 1360) ||
                     (pcId == 1475) )
                    ident = IDENT_EXT_BAS ;
                else
                    ident = IDENT_NEW_BAS ;

                nbSamp = (1 + 9) * 16 * 16 ;        /* Header = 10 Byte16 */
            }

            /* Footer = 3 Byte13                         */
            /* Lead = (nbByte + (nbByte / 120)) Byte13   */
            /* Byte = 16 or 13 bits, Bit = 16 Samples    */
            /* Sync = one Bit = 16 Samples (4000 Hz)     */

            nbSamp = nbSamp + ((nbByte + 3 + (nbByte / 120)) * 13 * 16) ;
            nbSamp = nbSamp + (nbSync * 16) ;
            break ;

        default :
            nbSamp = 0 ;
            ident  = 0 ;
            break ;
        }

        *ptrIdent = ident ;
        *ptrSamp  = nbSamp ;

    } while (0) ;
    return (error);
}


int WriteHeadToWav (u_long    nbSamp,
                    u_long    freq,
                    FileInfo* ptrFile)
{
    int     error ;

    do {
        error = WriteStringToFile ("RIFF", &ptrFile->ptrFd) ;
        if (error != ERR_OK) break ;

        error = WriteLongToFile ((nbSamp + 36), &ptrFile->ptrFd) ;
        if (error != ERR_OK) break ;

        error = WriteStringToFile ("WAVEfmt ", &ptrFile->ptrFd) ;
        if (error != ERR_OK) break ;

        error = WriteLongToFile (0x10, &ptrFile->ptrFd) ;
        if (error != ERR_OK) break ;

        error = WriteShortToFile (1, &ptrFile->ptrFd) ;      /* PCM */
        if (error != ERR_OK) break ;

        error = WriteShortToFile (1, &ptrFile->ptrFd) ;      /* Mono */
        if (error != ERR_OK) break ;

        error = WriteLongToFile (freq, &ptrFile->ptrFd) ;    /* Samp Freq */
        if (error != ERR_OK) break ;

        error = WriteLongToFile (freq, &ptrFile->ptrFd) ;    /* Byte / sec */
        if (error != ERR_OK) break ;

        error = WriteShortToFile (1, &ptrFile->ptrFd) ;      /* Byte / Samp x Chan */
        if (error != ERR_OK) break ;

        error = WriteShortToFile (8, &ptrFile->ptrFd) ;      /* Bit / Samp */
        if (error != ERR_OK) break ;

        error = WriteStringToFile ("data", &ptrFile->ptrFd) ;
        if (error != ERR_OK) break ;

        error = WriteLongToFile (nbSamp, &ptrFile->ptrFd) ;  /* Nb Samples */
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteBitToWav (u_long    value,
                   FileInfo* ptrFile)
{
    int     ii ;
    int     error ;

    for ( ii = 0 ; ii < 16 ; ii++ ) {
        error = fputc (bit[value][ii], ptrFile->ptrFd) ;
        if (error == EOF) {
            printf ("ERROR : Can't write in the file\n") ;
            error = ERR_NOK ;
            break ;
        }
        else
            error = ERR_OK ;
    }
    return (error);
}


int WriteSyncToWav (int       nbSync,
                    FileInfo* ptrFile)
{
    int     ii ;
    int     error ;

    do {
        /* Write the Synchro patern */
        for ( ii = 0 ; ii < nbSync ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteQuaterToWav (u_long    value,
                      u_long    nbBits,
                      FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < nbBits ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteToWav (u_long    value,
                    u_long    order,
                    u_long    mode,
                    FileInfo* ptrFile)
{
    u_long  lsq ;
    u_long  msq ;
    int     error ;

    do {
        lsq = value & 0x0F ;
        msq = (value >> 4) & 0x0F ;
        if (order == ORDER_INV) {
            error = WriteQuaterToWav (lsq, Mode[mode].nb1, ptrFile) ;
            if (error != ERR_OK) break ;
            error = WriteQuaterToWav (msq, Mode[mode].nb2, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        else {
            error = WriteQuaterToWav (msq, Mode[mode].nb1, ptrFile) ;
            if (error != ERR_OK) break ;
            error = WriteQuaterToWav (lsq, Mode[mode].nb2, ptrFile) ;
            if (error != ERR_OK) break ;
        }

    } while (0) ;
    return (error);
}


int WriteByteSumToWav (u_long    value,
                       u_long    order,
                       u_long    mode,
                       FileInfo* ptrFile)
{
    int     error ;

    do {
        error = WriteByteToWav (value, order, mode, ptrFile) ;
        if (error != ERR_OK) break ;

        ptrFile->count = ptrFile->count + 1;
        switch (mode) {
        case MODE_B22 :
            if ( (ptrFile->count % 80) == 0) {
            }
            break ;

        case MODE_B21 :
        case MODE_B20 :
        case MODE_B19 :
            if ( (ptrFile->count % 8) == 0) {
            }
            break ;

        case MODE_B17 :
        case MODE_B16 :
        case MODE_B14 :
        case MODE_B13 :
            if ( (ptrFile->count % 120) == 0) {
            }
            break ;

        default :
            printf ("ERROR : Unknown Mode\n") ;
            break ;
        }

    } while (0) ;
    return (error);
}


int WriteIdentToB22Wav (u_long    value,
                        FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 6 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteToB22Wav (u_long    value,
                       FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 6 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 4 ; ii < 8 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 6 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteToB19Wav (u_long    value,
                       FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 4 ; ii < 8 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 5 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteToB16Wav (u_long    value,
                       FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 4 ; ii < 8 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        error = WriteBitToWav (1, ptrFile) ;
        if (error != ERR_OK) break ;
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 5 ; ii++ ) {
            error = WriteBitToWav (1, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteToB13Wav (u_long    value,
                       FileInfo* ptrFile)
{
    u_long  tmp ;
    int     ii ;
    int     error ;

    do {
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 4 ; ii < 8 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        error = WriteBitToWav (1, ptrFile) ;
        if (error != ERR_OK) break ;
        error = WriteBitToWav (0, ptrFile) ;
        if (error != ERR_OK) break ;

        for ( ii = 0 ; ii < 4 ; ii++ ) {
            tmp = 1 << ii ;
            if ( (value & tmp) == 0 )
                error = WriteBitToWav (0, ptrFile) ;
            else
                error = WriteBitToWav (1, ptrFile) ;

            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        error = WriteBitToWav (1, ptrFile) ;
        if (error != ERR_OK) break ;
        error = WriteBitToWav (1, ptrFile) ;
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteByteSumToB22Wav (u_long    value,
                          FileInfo* ptrFile)
{
    u_long  cnt ;
    u_long  sum ;
    u_long  tmpL ;
    int     error ;

    do {
        cnt = ptrFile->count ;
        sum = ptrFile->sum ;

        /* Write the byte */
        error = WriteByteToB22Wav (value, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Update the SumCheck and the Byte Counter */
        sum = sum + value ;
        cnt = cnt + 1 ;

        if (cnt >= 80) {
            /* Write the SumCheck */
            tmpL = (sum >> 8) & 0xFF ;
            error = WriteByteToB22Wav (tmpL, ptrFile) ;
            if (error != ERR_OK) break ;

            tmpL = sum & 0xFF ;
            error = WriteByteToB22Wav (tmpL, ptrFile) ;
            if (error != ERR_OK) break ;

            cnt = 0 ;
            sum = 0 ;
        }

        ptrFile->count = cnt ;
        ptrFile->sum   = sum ;

    } while (0) ;
    return (error);
}


int WriteByteSumToB19Wav (u_long    value,
                          FileInfo* ptrFile)
{
    u_long  cnt ;
    u_long  sum ;
    int     error ;

    do {
        cnt = ptrFile->count ;
        sum = ptrFile->sum ;

        /* Write the byte */
        error = WriteByteToB19Wav (value, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Update the SumCheck */
        sum = sum + ((value & 0xF0) >> 4) ;
        if (sum > 0xFF)
            sum = (sum + 1) & 0xFF ;
        sum = (sum + (value & 0x0F)) & 0xFF ;

        /* Update the Byte Counter */
        cnt = cnt + 1 ;

        if ( (cnt % 8) == 0 ) {
            /* Write the SumCheck */
            error = WriteByteToB19Wav (sum, ptrFile) ;
            if (error != ERR_OK) break ;

            if (cnt >= 80) {
                cnt = 0 ;
                sum = 0 ;
            }
        }

        ptrFile->count = cnt ;
        ptrFile->sum   = sum ;

    } while (0) ;
    return (error);
}


int WriteByteSumToB16Wav (u_long    value,
                          FileInfo* ptrFile)
{
    u_long  cnt ;
    u_long  sum ;
    int     error ;

    do {
        cnt = ptrFile->count ;
        sum = ptrFile->sum ;

        /* Write the byte */
        error = WriteByteToB16Wav (value, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Update the SumCheck */
        sum = sum + ((value & 0xF0) >> 4) ;
        if (sum > 0xFF)
            sum = (sum + 1) & 0xFF ;
        sum = (sum + (value & 0x0F)) & 0xFF ;

        /* Update the Byte Counter */
        cnt = cnt + 1 ;

        if (cnt >= 8) {
            /* Write the SumCheck */
            error = WriteByteToB16Wav (sum, ptrFile) ;
            if (error != ERR_OK) break ;

            cnt = 0 ;
            sum = 0 ;
        }

        ptrFile->count = cnt ;
        ptrFile->sum   = sum ;

    } while (0) ;
    return (error);
}


int WriteByteSumToB13Wav (u_long    value,
                          FileInfo* ptrFile)
{
    u_long  cnt ;
    u_long  sum ;
    int     error ;

    do {
        cnt = ptrFile->count ;
        sum = ptrFile->sum ;

        /* Write the byte */
        error = WriteByteToB13Wav (value, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Update the SumCheck */
        sum = sum + ((value & 0xF0) >> 4) ;
        if (sum > 0xFF)
            sum = (sum + 1) & 0xFF ;
        sum = (sum + (value & 0x0F)) & 0xFF ;

        /* Update the Byte Counter */
        cnt = cnt + 1 ;

        if (cnt >= 120) {
            /* Write the SumCheck */
            error = WriteByteToB13Wav (sum, ptrFile) ;
            if (error != ERR_OK) break ;

            cnt = 0 ;
            sum = 0 ;
        }

        ptrFile->count = cnt ;
        ptrFile->sum   = sum ;

    } while (0) ;
    return (error);
}


int WriteHeadToB22Wav (char*     ptrName,
                       u_long    addr,
                       u_long    size,
                       u_long    type,
                       FileInfo* ptrFile)
{
    int     ii ;
    u_long  len ;
    u_long  tmpL ;
    char    tmpS[20] ;
    int     error ;

    do {
        /* Search the length */
        tmpL = strlen (ptrName) ;
        if (tmpL > 16)
            tmpL = 16 ;

        for ( ii = 0 ; ii < tmpL ; ii++ )
            tmpS[ii] = ptrName[ii] ;
        for ( ii = tmpL ; ii < 16 ; ii++ )
            tmpS[ii] = 0 ;

        printf ("Save name     : %s\n", tmpS) ;

        /* Write the Header */
        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;
        for ( ii = 0x10 ; ii < 0x18 ; ii++ ) {
            error = WriteByteSumToB22Wav (ii, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        /* Write the first Sub-Ident */
        if (type == TYPE_BIN)
            tmpL = 0x00 ;
        else
            tmpL = 0x01 ;

        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the Name */
        for ( ii = 0 ; ii < 16 ; ii++ ) {
            error = WriteByteSumToB22Wav (tmpS[ii], ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        /* Write 9 null bytes */
        for ( ii = 0 ; ii < 9 ; ii++ ) {
            error = WriteByteSumToB22Wav (0, ptrFile) ;
            if (error != ERR_OK) break ;
        }
        if (error != ERR_OK) break ;

        /* Write the Addresse */
        tmpL = (addr >> 8) & 0xFF ;
        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = addr & 0xFF ;
        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the Buffer Size */
        if (type == TYPE_BIN)
            len = size - 1 ;
        else
            len = size ;

        tmpL = (len >> 8) & 0xFF ;
        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = len & 0xFF ;
        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the Sub-Ident */
        if (type == TYPE_BIN)
            tmpL = 0xFF ;
        else
            tmpL = 0x00 ;

        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteByteSumToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the SumCheck */
        tmpL = (ptrFile->sum >> 8) & 0xFF ;
        error = WriteByteToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = ptrFile->sum & 0xFF ;
        error = WriteByteToB22Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteSyncToWav (70, ptrFile) ;
        if (error != ERR_OK) break ;

        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;

    } while (0) ;
    return (error);
}


int WriteSaveNameToB19Wav (char*     ptrName,
                           FileInfo* ptrFile)
{
    int     ii ;
    u_long  byte ;
    u_long  tmpL ;
    char*   ptrDot ;
    char    tmpS[10] ;
    int     error ;

    do {
        /* Upercase the Name */
        (void) strupr (ptrName) ;
        tmpL = strlen (ptrName) ;
        if (tmpL > 7)
            tmpL = 7 ;

        for ( ii = 0 ; ii < tmpL ; ii++ )
            tmpS[ii] = ptrName[ii] ;
        tmpS[tmpL] = 0 ;
        printf ("Save name     : %s\n", tmpS) ;

        tmpL = 7 - tmpL ;
        for ( ii = 0 ; ii < tmpL ; ii++ )
            tmpS[ii] = 0 ;

        for ( ii = tmpL ; ii < 7 ; ii++ ) {
            byte = (u_long) ptrName[6 - ii] ;
            if (byte < 0x80)
                byte = CodeOld[byte] ;
            else
                byte = CodeOld[0] ;

            tmpS[ii] = (char) ( (byte >> 4) + ((byte << 4) & 0xF0) ) ;
        }
        tmpS[7] = 0x5F ;

        /* Write the Name */
        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;
        for ( ii = 0 ; ii < 8 ; ii++ ) {
            error = WriteByteSumToB19Wav (tmpS[ii], ptrFile) ;
            if (error != ERR_OK) break ;
        }
        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;

    } while (0) ;
    return (error);
}


int WriteSaveNameToB16Wav (char*     ptrName,
                           FileInfo* ptrFile)
{
    int     ii ;
    u_long  byte ;
    u_long  tmpL ;
    char*   ptrDot ;
    char    tmpS[10] ;
    int     error ;

    do {
        /* Search the Dot position */
        tmpL = strlen (ptrName) ;
        if (tmpL > 7)
            tmpL = 7 ;

        for ( ii = 0 ; ii < tmpL ; ii++ )
            tmpS[ii] = ptrName[ii] ;
        tmpS[tmpL] = 0 ;
        printf ("Save name     : %s\n", tmpS) ;

        tmpL = 7 - tmpL ;
        for ( ii = 0 ; ii < tmpL ; ii++ )
            tmpS[ii] = 0 ;

        for ( ii = tmpL ; ii < 7 ; ii++ ) {
            byte = (u_long) ptrName[6 - ii] ;
            if (byte >= 0x80)
                byte = 0x20 ;

            tmpS[ii] = (char) ( (byte >> 4) + ((byte << 4) & 0xF0) ) ;
        }
        tmpS[7] = 0x5F ;

        /* Write the Name */
        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;
        for ( ii = 0 ; ii < 8 ; ii++ ) {
            error = WriteByteSumToB16Wav (tmpS[ii], ptrFile) ;
            if (error != ERR_OK) break ;
        }
        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;

    } while (0) ;
    return (error);
}


int WriteHeadToB19BinWav (u_long    addr,
                          u_long    size,
                          FileInfo* ptrFile)
{
    int     ii ;
    u_long  len ;
    u_long  tmpL ;
    int     error ;

    do {
        printf ("Start Address : 0x%04X\n", addr);
        printf ("Buffer Length : %d bytes\n", size);

        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;
        for ( ii = 0 ; ii < 4 ; ii++ ) {
            error = WriteByteSumToB19Wav (0, ptrFile) ;
            if (error != ERR_OK) break ;
        }

        /* Write the Addresse */
        tmpL = ((addr >> 4) & 0xF0) + (addr >> 12) ;
        error = WriteByteSumToB19Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = ((addr << 4) & 0xF0) + ((addr >> 4) & 0x0F) ;
        error = WriteByteSumToB19Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the Length */
        len = size - 1 ;
        tmpL = ((len >> 4) & 0xF0) + (len >> 12) ;
        error = WriteByteSumToB19Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = ((len << 4) & 0xF0) + ((len >> 4) & 0x0F) ;
        error = WriteByteSumToB19Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;

    } while (0) ;
    return (error);
}


int WriteHeadToB16BinWav (u_long    addr,
                          u_long    size,
                          FileInfo* ptrFile)
{
    int     ii ;
    u_long  len ;
    u_long  tmpL ;
    int     error ;

    do {
        printf ("Start Address : 0x%04X\n", addr);
        printf ("Buffer Length : %d bytes\n", size);

        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;
        for ( ii = 0 ; ii < 4 ; ii++ ) {
            error = WriteByteSumToB16Wav (0, ptrFile) ;
            if (error != ERR_OK) break ;
        }

        /* Write the Addresse */
        tmpL = ((addr >> 4) & 0xF0) + (addr >> 12) ;
        error = WriteByteSumToB16Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = ((addr << 4) & 0xF0) + ((addr >> 4) & 0x0F) ;
        error = WriteByteSumToB16Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        /* Write the Length */
        len = size - 1 ;
        tmpL = ((len >> 4) & 0xF0) + (len >> 12) ;
        error = WriteByteSumToB16Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        tmpL = ((len << 4) & 0xF0) + ((len >> 4) & 0x0F) ;
        error = WriteByteSumToB16Wav (tmpL, ptrFile) ;
        if (error != ERR_OK) break ;

        ptrFile->count = 0 ;
        ptrFile->sum   = 0 ;

    } while (0) ;
    return (error);
}


int WriteFooterToB22BinWav (u_long    type,
                            FileInfo* ptrFile)
{
    u_long  sum ;
    int     error ;

    do {
        ptrFile->count = 0 ;
        if (type != TYPE_BIN) {
            error = WriteByteSumToB22Wav (0xFF, ptrFile) ;
            if (error != ERR_OK) break ;
        }

        sum = (ptrFile->sum >> 8) & 0xFF ;
        error = WriteByteToB22Wav (sum, ptrFile) ;
        if (error != ERR_OK) break ;

        sum = ptrFile->sum & 0xFF ;
        error = WriteByteToB22Wav (sum, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteSyncToWav (70, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteByteToB22Wav (0x55, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteSyncToWav (70, ptrFile) ;
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int WriteFooterToB13BinWav (FileInfo* ptrFile)
{
    u_long  sum ;
    int     error ;

    do {
        ptrFile->count = 0 ;
        error = WriteByteSumToB13Wav (0xFF, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteByteToB13Wav (0xFF, ptrFile) ;
        if (error != ERR_OK) break ;

        error = WriteByteToB13Wav (ptrFile->sum, ptrFile) ;
        if (error != ERR_OK) break ;

    } while (0) ;
    return (error);
}


int ConvertBinToWav (char*   ptrSrcFile,
                     char*   ptrDstFile,
                     u_long  type,
                     u_long  pcId,
                     u_long  addr,
                     u_long  nbSync,
                     char*   ptrName)
{
    FileInfo    info ;
    FILE        *srcFd ;
    int         inVal ;
    int         ii ;
    u_long      nbByte ;
    u_long      nbSamp ;
    u_long      freq ;
    u_long      sync ;
    int         error ;

    do {
        info.ptrFd  = NULL ;
        info.ident  = IDENT_UNKNOWN ;
        info.count  = 0 ;
        info.sum    = 0 ;
        srcFd       = NULL ;
        error       = ERR_OK ;

        /* Open the destination file */
        info.ptrFd = fopen (ptrDstFile, "wb") ;
        if (info.ptrFd == NULL) {
            printf ("ERROR : Can't open the destination file : %s\n",
                    ptrDstFile) ;
            error = ERR_NOK ;
            break ;
        }

        /* Open the source file */
        srcFd = fopen (ptrSrcFile, "rb") ;
        if (srcFd == NULL) {
            printf ("ERROR : Can't open the source file : %s\n",
                    ptrSrcFile) ;
            error = ERR_NOK ;
            break ;
        }

        error = ReadFileLength (&nbByte, &srcFd) ;
        if (error != ERR_OK) break ;

        if (nbByte > 0x10000) {
            printf ("ERROR : Source file contains more than 65536 bytes\n") ;
            error = ERR_NOK ;
            break ;
        }

        if ( (addr + nbByte) > 0x10000) {
            printf ("ERROR : (Address + Size) greater than 65536)\n") ;
            error = ERR_NOK ;
            break ;
        }

        if (pcId == 1500)
            freq = BASE_FREQ2 ;
        else
            freq = BASE_FREQ1 ;

        sync = (nbSync * freq) / 8 ;

        /* Calculate the length and Ident of the destination file */
        error = LengthAndIdentOfBinWav (type, pcId, nbByte, sync, &info.ident, &nbSamp) ;
        if (error != ERR_OK) break ;

        /* Write the header of the destination WAV file */
        error = WriteHeadToWav (nbSamp, (freq *2), &info) ;
        if (error != ERR_OK) break ;

        /* Write the synchro patern for N times */
        printf ("Synchro size  : %d bits\n", sync);
        error = WriteSyncToWav (sync, &info) ;
        if (error != ERR_OK) break ;

        printf ("Wave format   : 0x%02X <- ", info.ident) ;
        if (type == TYPE_BIN)
            printf ("Binary, PC-%d\n", pcId) ;
        else
            printf ("Basic, PC-%d\n", pcId) ;

        if (pcId == 1500) {
            /* Write the TAP code */
            error = WriteIdentToB22Wav (IDENT_PC1500, &info) ;
            if (error != ERR_OK) break ;

            /* Write the Name */
            error = WriteHeadToB22Wav (ptrName, addr, nbByte, type, &info) ;
            if (error != ERR_OK) break ;

            if (type == TYPE_BIN) {
                printf ("Start Address : 0x%04X\n", addr);
                printf ("Buffer Length : %d bytes\n", nbByte);
            }
        }

        if ( (info.ident == IDENT_OLD_BAS) ||
             (info.ident == IDENT_OLD_BIN) ) {
            /* Write the TAP code */
            error = WriteByteToB19Wav (info.ident, &info) ;
            if (error != ERR_OK) break ;

            /* Write the Name */
            error = WriteSaveNameToB19Wav (ptrName, &info) ;
            if (error != ERR_OK) break ;
        }

        if ( (info.ident == IDENT_NEW_BAS) ||
             (info.ident == IDENT_EXT_BAS) ||
             (info.ident == IDENT_NEW_BIN) ) {
            /* Write the TAP code */
            error = WriteByteToB16Wav (info.ident, &info) ;
            if (error != ERR_OK) break ;

            /* Write the Name */
            error = WriteSaveNameToB16Wav (ptrName, &info) ;
            if (error != ERR_OK) break ;
        }

        switch (info.ident) {
        case IDENT_PC15_BAS :
            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB22Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteFooterToB22BinWav (type, &info) ;
            break ;

        case IDENT_PC15_BIN :
            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB22Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteFooterToB22BinWav (type, &info) ;
            break ;

        case IDENT_OLD_BAS :
            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB19Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteByteToB19Wav (0xF0, &info) ;
            break ;

        case IDENT_OLD_BIN :
            /* Write the addresse and length */
            error = WriteHeadToB19BinWav (addr, nbByte, &info) ;
            if (error != ERR_OK) break ;

            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB19Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteByteToB19Wav (0xF0, &info) ;

            break ;

        case IDENT_NEW_BAS :
        case IDENT_EXT_BAS :
            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB13Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteFooterToB13BinWav (&info) ;
            break ;

        case IDENT_NEW_BIN :
            /* Write the addresse and length */
            error = WriteHeadToB16BinWav (addr, nbByte, &info) ;
            if (error != ERR_OK) break ;

            /* Write the datas */
            do {
                inVal = fgetc (srcFd) ;
                if (inVal == EOF) break ;

                error = WriteByteSumToB13Wav (inVal, &info) ;
                if (error != ERR_OK) break ;

            } while (1) ;
            if (error != ERR_OK) break ;

            /* Write the END code */
            error = WriteFooterToB13BinWav (&info) ;
            break ;

        default :
            printf ("ERROR : Unknown Ident\n") ;
            break ;
        }

    } while (0) ;

    /* Close the source file */
    if (srcFd != NULL) {
        error = fclose (srcFd) ;
        if (error != ERR_OK) {
            printf ("ERROR : Can't close the source file\n") ;
        }
    }

    /* Close the destination file */
    if (info.ptrFd != NULL) {
        error = fclose (info.ptrFd) ;
        if (error != ERR_OK) {
            printf ("ERROR : Can't close the destination file\n") ;
        }
    }

    return (error);
}


void PrintHelp (void)
{
    printf ("\n") ;
    printf ("Usage   : BIN2WAV srcfile dstfile %stype %snum [%saddr] [%ssync] [%sname]\n",
             Token[0], Token[1], Token[2], Token[3], Token[4]) ;
    printf ("\n") ;
    printf ("srcfile : Source file name\n") ;
    printf ("dstfile : Destination file name (NULL for none)\n") ;
    printf ("type    : Source file type :\n") ;
    printf ("           BIN -> Binary, IMG -> Basic binary image.\n") ;
    printf ("num     : Sharp pocket computer, currently available :\n") ;
    printf ("           1245, 1251, 1261, 1350, 1360, 1401, 1402, 1403\n") ;
    printf ("           1450, 1475, 1500\n") ;
    printf ("addr    : Start addresse, needed for BIN type :\n") ;
    printf ("           0 to 65535, or 0x0000 to 0xFFFF (default = 0)\n") ;
    printf ("sync    : Sync length, 1 to 10 second (default = 2)\n") ;
    printf ("name    : File name (default = dstfile)\n") ;
    printf ("\n") ;
    printf ("Version : 1.30\n") ;
    printf ("Author  : Pocket -> www.pocketmuseum.com\n") ; /* do not remove please */
}

void main (int argc, char* argv[])
{
    char    argS[8][100] = { "", "", "", "", "", "", "", "" } ;
    char    argD[8][100] = { "", "", "", "", "0x0000", "2", "", "" } ;
    int     ii ;
    int     jj ;
    int     nbArg ;
    char    *ptrToken ;
    char    *ptrErr ;
    char    tmp ;
    u_long  type ;
    u_long  pcId ;
    u_long  sync ;
    long    addr ;
    int     error ;

    do {
        error = ERR_OK ;
        addr  = 0 ;

        /* Check if the number if arguments is correct */
        if ( (argc < 5) || (argc > 8) ) {
            PrintHelp() ;
            break ;
        }

        /* Extract the source file name */
        (void) strcpy (argD[0], argv[1]) ;

        /* Extract the destination file name */
        (void) strcpy (argS[1], argv[2]) ;
        (void) strupr (argS[1]) ;
        if (strcmp (argS[1], "NULL") == 0)
            (void) strcpy (argD[1], argS[1]) ;
        else
            (void) strcpy (argD[1], argv[2]) ;

        /* Extract the arguments */
        nbArg = argc - 3 ;
        for ( ii = 0 ; ii < nbArg ; ii++ ) {
            (void) strcpy (argS[ii], argv[ii + 3]) ;
            (void) strupr (argS[ii]) ;
        }

        for ( ii = 0 ; ii < nbArg ; ii++ ) {
            for ( jj = 0 ; jj < TOKEN_NB ; jj++ ) {
                ptrToken = strstr (argS[ii], Token[jj]) ;
                if (ptrToken == argS[ii]) {
                    (void) strcpy (argD[jj+2], argv[ii+3] + strlen (Token[jj])) ;
                    break ;
                }
            }
        }
        (void) strupr (argD[2]) ;

        /* Compare the Type to the allowed tokens */
        if (strlen (argD[2]) == 0) {
            printf ("ERROR : No source file type specified '%s'\n", Token[0]) ;
            PrintHelp ();
            break ;
        }

        type = TYPE_NOK ;
        if (strcmp (argD[2], "BIN") == 0)
            type = TYPE_BIN ;

        if (strcmp (argD[2], "IMG") == 0)
            type = TYPE_IMG ;

        if (type == TYPE_NOK) {
            printf ("ERROR : Source file type '%s%s' is not valid.\n", Token[0], argD[2]) ;
            PrintHelp ();
            break ;
        }

        /* Compare the PC Ident to the allowed tokens */
        if (strlen (argD[3]) == 0) {
            printf ("ERROR : No pocket computer specified '%s'\n", Token[1]) ;
            PrintHelp ();
            break ;
        }
        pcId = strtol (argD[3], &ptrErr, 0) ;
        if (pcId == 0) {
            printf ("ERROR : Pocket computer '%s%s' is not valid.\n", Token[1], argD[3]) ;
            PrintHelp ();
            break ;
        }

        switch (pcId) {
        case 1245 :
        case 1251 :
        case 1261 :
        case 1350 :
        case 1360 :
        case 1401 :
        case 1402 :
        case 1403 :
        case 1450 :
        case 1475 :
        case 1500 :
            break ;

        default :
            printf ("ERROR : Pocket computer %s%s not implemented.\n", Token[1], argD[3]) ;
            PrintHelp ();
            error = ERR_NOK ;
            break ;
        }
        if (error != ERR_OK) break ;

        /* Convert the Address in a long if necessary */
        if (type == TYPE_BIN) {
            addr = strtol (argD[4], &ptrErr, 0) ;
            if (*ptrErr != NULL) {
                tmp = *ptrErr ;
                *ptrErr = 0 ;
                printf ("ERROR : Start address is not valid '%s%s -> ", Token[2], argD[4]) ;
                *ptrErr = tmp ;
                printf ("%s'\n", ptrErr) ;
                PrintHelp ();
                break ;
            }

            if ( (addr < 0) ||
                 (addr > 0xFFFF) ) {
                printf ("ERROR : Start address '%s' is out of range\n", argD[4]) ;
                PrintHelp ();
                break ;
            }
        }

        /* Convert the Sync length in a long */
        sync = strtol (argD[5], &ptrErr, 0) ;
        if (*ptrErr != NULL) {
            tmp = *ptrErr ;
            *ptrErr = 0 ;
            printf ("ERROR : Sync length is not valid '%s%s -> ", Token[3], argD[5]) ;
            *ptrErr = tmp ;
            printf ("%s'\n", ptrErr) ;
            PrintHelp ();
            break ;
        }

        if ( (sync < 1) ||
             (sync > 10) ) {
            printf ("ERROR : Sync length '%s' is out of range\n", argD[5]) ;
            PrintHelp ();
            break ;
        }

        /* Check if the name is not defined */
        if (strlen (argD[6]) == 0) {
            /* Search the last colon position */
            ptrToken = strrchr (argD[1], CHAR_COLON) ;
            if (ptrToken == NULL)
                (void) strcpy (argS[6], argD[1]) ;
            else
                (void) strcpy (argS[6], ptrToken + 1) ;

            /* Search the last slash position */
            ptrToken = strrchr (argS[6], CHAR_SLASH) ;
            if (ptrToken == NULL)
                (void) strcpy (argD[6], argS[6]) ;
            else
                (void) strcpy (argD[6], ptrToken + 1) ;

            /* Search the dot position */
            ptrToken = strchr (argD[6], CHAR_DOT) ;
            if (ptrToken != NULL)
                *ptrToken = 0 ;
        }

        /* Replace the underscore with space in the name */
        ptrToken = strchr (argD[6], CHAR_UNDERSCORE) ;
        while (ptrToken != NULL) {
            *ptrToken = CHAR_SPACE ;
            ptrToken = strchr (argD[6], CHAR_UNDERSCORE) ;
        }

        /* Upercase the filenames */
        (void) strupr (argD[0]) ;
        (void) strupr (argD[1]) ;

        /* Call the Convert function */
        (void) ConvertBinToWav (argD[0], argD[1], type, pcId, addr, sync, argD[6]) ;

    } while (0) ;
}